package roneditor.actions;

import java.util.ArrayList;
import java.util.HashMap;

import org.eclipse.gef.ui.actions.SelectionAction;

import roneditor.converter.RuleSimulator;
import roneditor.dialogs.AbstractMatchDialog;
import roneditor.dialogs.MatchDialog;
import roneditor.edit.parts.RONTreeEditPart;
import roneditor.editor.RONEditor;
import ronmodel.Codomain;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RON;
import ronmodel.Rule;

/**
 * This is a sample action to demonstrate AbstractMatchDialog. It is enabled if
 * the RON in the background of the tree viewer is selected. It just grabs all
 * rules and object nets in the RON and puts the result of the rule application
 * on the HLPlace that hosts the selected object net.
 * 
 * @author Tony Modica
 */
public class ApplyRuleAction extends SelectionAction {

	public static final String ID = "roneditor.actions.ApplyRuleAction";

	public static final String NAME = "Match Dialog";

	public ApplyRuleAction(final RONEditor part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		final RON ron = ((RONTreeEditPart) getSelectedObjects().get(0))
				.getCastedModel();

		// get all rules and object nets in the RON
		final ArrayList<Rule> rules = new ArrayList<Rule>();
		final ArrayList<ObjectNet> objectNets = new ArrayList<ObjectNet>();

		for (final Object node : ron.getNodes()) {
			if (node instanceof HLPlace) {
				final HLPlace place = (HLPlace) node;
				if (place.getType().equals(HLPlaceType.NET_LITERAL)) {
					objectNets.addAll(place.getTokens());
				} else if (place.getType().equals(HLPlaceType.RULE_LITERAL)) {
					rules.addAll(place.getTokens());
				}
			}
		}

		if (objectNets.isEmpty() || rules.isEmpty()) {
			// nothing to do
			return;
		}

		// open dialog
		final AbstractMatchDialog dialog = new MatchDialog(rules, objectNets);
		dialog.open();
		if (dialog.isCanceled()) {
			return;
		}

		// fetch information from the dialog
		final Rule rule = dialog.getCurrentRule();
		final Codomain objectNet = dialog.getObjectNetCopy();
		final HLPlace parentPlace = dialog.getHLPlace();
		final HashMap<ONNode, ONNode> match = dialog.getMatchToApply();

		// apply the selected rule to the selected object net
		(new RuleSimulator()).applyRule(match, rule, objectNet);
		objectNet.setName(objectNet.getName() + "Trans");

		// put the result on the corresponding HLPlace
		parentPlace.getTokens().add(objectNet);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link RONTreeEditPart}.
	 */
	@Override
	protected boolean calculateEnabled() {
		return getSelection() != null && !getSelection().isEmpty()
				&& getSelectedObjects().get(0) instanceof RONTreeEditPart;
	}

}
