package roneditor.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;

import roneditor.dialogs.CreateHLPlaceOrTokenDialog;
import roneditor.editor.RONEditor;
import roneditor.ronview.commands.CreateHLPlaceCommand;
import ronmodel.HLPlaceType;

public class CreateHLPlaceAction extends SelectionAction {

	static public final String ID = "Create HLPlace";

	public CreateHLPlaceAction(final RONEditor part) {
		super(part);
		setId(CreateHLPlaceAction.ID);
		setText("Create HLPlace");
		setDescription("Create a place in the RON");
		setToolTipText("Create a place in the RON");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		// create dialog
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final CreateHLPlaceOrTokenDialog dialog = new CreateHLPlaceOrTokenDialog(
				shell, SWT.NULL) {
			@Override
			protected void init() {
				// configure dialog
				getDialogShell().setText("Create place");
				getNetRadioButton().setEnabled(true);
				getNetRadioButton().setSelection(true);
				getRuleRadioButton().setEnabled(true);
				getRuleRadioButton().setSelection(false);
				getNameText().setText("newplace");
				getNameText().setFocus();
			}
		};

		// open dialog
		dialog.open();
		if (dialog.isCancelled()) {
			return;
		}

		// fill command from editpart with dialog data
		final CreateHLPlaceCommand command = getCommand();
		command.setName(dialog.getNameTextValue());
		if (dialog.isNetType()) {
			command.setType(HLPlaceType.NET_LITERAL);
		} else {
			command.setType(HLPlaceType.RULE_LITERAL);
		}
		execute(command);
	}

	/**
	 * Gets the command from the selected edit part which delivers a Command for
	 * the ID request since {@link #calculateEnabled()} must have returned true.
	 */
	private CreateHLPlaceCommand getCommand() {
		// HINT Die Action besorgt sich ein Command vom ausgewhlten Editpart
		final Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();
		return (CreateHLPlaceCommand) ((EditPart) selection)
				.getCommand(new Request(ID));
	}

	/**
	 * Calculates whether this action is enabled or not by checking if it
	 * understands the ID request.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		// HINT Diese Action soll nur fr genau einen ausgewhlten EditPart
		// angezeigt
		// werden, der auch den Request ID versteht
		if (getSelectedObjects().size() == 1
				&& (getSelectedObjects().get(0) instanceof EditPart)) {
			final EditPart part = (EditPart) getSelectedObjects().get(0);
			return part.understandsRequest(new Request(ID));
		}
		return false;
	}
}
