package roneditor.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;

import roneditor.dialogs.CreateHLPlaceOrTokenDialog;
import roneditor.editor.RONEditor;
import roneditor.ronview.commands.CreateTokenCommand;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;

/**
 * An action for the RON editor to create tokens (i.e. low-level nets or rules)
 * on places in a RON.
 * 
 * @author Tony Modica
 */
public class CreateTokenAction extends SelectionAction {

	static public final String ID = "Create Token";

	public CreateTokenAction(final RONEditor part) {
		super(part);
		setId(CreateTokenAction.ID);
		setText("Create Token");
		setDescription("Create token on the place");
		setToolTipText("Create token on the place");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	@Override
	public void run() {
		// create dialog
		final Shell shell = getWorkbenchPart().getSite().getShell();
		final CreateHLPlaceOrTokenDialog dialog = new CreateHLPlaceOrTokenDialog(
				shell, SWT.NULL) {
			@Override
			protected void init() {
				// configure dialog
				getDialogShell().setText("Create token");
				getNetRadioButton().setEnabled(false);
				getRuleRadioButton().setEnabled(false);

				final Object selection = ((StructuredSelection) getSelection())
						.getFirstElement();
				final HLPlace place = (HLPlace) ((EditPart) selection)
						.getModel();
				if (place.getType().equals(HLPlaceType.NET_LITERAL)) {
					getNetRadioButton().setSelection(true);
					getRuleRadioButton().setSelection(false);
				} else {
					getRuleRadioButton().setSelection(true);
					getNetRadioButton().setSelection(false);
				}

				getNameText().setText("newToken");
				getNameText().setFocus();
			}
		};

		// open dialog
		dialog.open();
		if (dialog.isCancelled()) {
			return;
		}

		// fill command from editpart with dialog data
		final CreateTokenCommand command = getCommand();
		command.setName(dialog.getNameTextValue());
		execute(command);
	}

	/**
	 * Gets the command from the selected edit part which delivers a Command for
	 * the ID request since {@link #calculateEnabled()} must have returned true.
	 */
	private CreateTokenCommand getCommand() {
		final Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();
		return (CreateTokenCommand) ((EditPart) selection)
				.getCommand(new Request(ID));
	}

	/**
	 * Calculates whether this action is enabled or not by checking if it
	 * understands the ID request.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	@Override
	protected boolean calculateEnabled() {
		if (getSelectedObjects().size() == 1
				&& (getSelectedObjects().get(0) instanceof EditPart)) {
			final EditPart part = (EditPart) getSelectedObjects().get(0);
			return part.understandsRequest(new Request(ID));
		}
		return false;
	}
}
