package roneditor.actions;

import java.util.ArrayList;

import org.eclipse.gef.ui.actions.SelectionAction;

import roneditor.dialogs.FireDialog;
import roneditor.edit.parts.RONTreeEditPart;
import roneditor.editor.RONEditor;
import roneditor.objectnetview.commands.FireONTransitionCommand;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RON;

/**
 * This is a sample action to demonstrate AbstractFireDialog. It is enabled if
 * the RON in the background of the tree viewer is selected. It just grabs all
 * object nets in the RON and fires the selected transition.
 * 
 * @author Tony Modica
 */
public class FireAction extends SelectionAction {

	public static final String ID = "roneditor.actions.FireAction";

	public static final String NAME = "Fire Dialog";

	public FireAction(final RONEditor part) {
		super(part);
		setId(ID);
		setText(NAME);
	}

	@Override
	public void run() {
		final RON ron = ((RONTreeEditPart) getSelectedObjects().get(0))
				.getCastedModel();

		// get all object nets in the RON
		final ArrayList<ObjectNet> objectNets = new ArrayList<ObjectNet>();

		for (final Object node : ron.getNodes()) {
			if (node instanceof HLPlace) {
				final HLPlace place = (HLPlace) node;
				if (place.getType().equals(HLPlaceType.NET_LITERAL)) {
					objectNets.addAll(place.getTokens());
				}
			}
		}

		if (objectNets.isEmpty()) {
			// nothing to do
			return;
		}

		// open dialog
		final FireDialog dialog = new FireDialog(objectNets);
		dialog.open();
		if (dialog.isCanceled()) {
			return;
		}

		// fetch information from the dialog
		final ONTransition transition = dialog.getTransitionToFire();

		// TODO fire the transition
		FireONTransitionCommand command = new FireONTransitionCommand();
		command.setOnTransition(transition);
		execute(command);
	}

	/**
	 * The {@link SelectionAction} is enabled if the first selected object is a
	 * {@link RONTreeEditPart}.
	 */
	@Override
	protected boolean calculateEnabled() {
		return getSelection() != null && !getSelection().isEmpty()
				&& getSelectedObjects().get(0) instanceof RONTreeEditPart;
	}

}
