package roneditor.edit.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;

import roneditor.commands.RenameNamedElementCommand;
import ronmodel.NamedElement;

/**
 * An EditPolicy for EditParts of NamedElement models to handle
 * DirectEditRequests.
 * 
 * @author Tony Modica
 */
public class NamedElementDirectEditPolicy extends DirectEditPolicy {

	/**
	 * @return A RenameNamedElementCommand with the NamedElement which is the
	 *         model of the host EditPart and the new name included in the
	 *         request set.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.DirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
	 */
	@Override
	protected Command getDirectEditCommand(final DirectEditRequest request) {
		final RenameNamedElementCommand command = new RenameNamedElementCommand(
				(NamedElement) getHost().getModel());
		// Das DirectEditFeature wird im TreeDirectEditManager zum
		// bertragen des Namens missbraucht!
		command.setNewName((String) request.getDirectEditFeature());
		return command;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
	 */
	@Override
	protected void showCurrentEditValue(@SuppressWarnings("unused")
	final DirectEditRequest request) {
	}

}
