package roneditor.figures;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.NodeEditPart;

/**
 * ConnectionAnchor fr Verbindung zwischen 2 Figures, die mit einer Connection
 * Verbunden sind
 * 
 * @see NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
 * @see NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
 * @see NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
 * @see NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
 */
public class NodeConnectionAnchor extends AbstractConnectionAnchor {
	protected int offsetH;

	protected int offsetV;

	protected int id;

	public NodeConnectionAnchor(IFigure owner, int id) {
		super(owner);
		this.id = id;
	}

	/**
	 * @see org.eclipse.draw2d.AbstractConnectionAnchor#ancestorMoved(IFigure)
	 */
	public void ancestorMoved(IFigure figure) {
		if (figure instanceof ScalableFigure)
			return;
		super.ancestorMoved(figure);
	}

	/**
	 * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
	 */
	public Point getLocation(Point reference) {
		Rectangle r = getOwner().getBounds();
		int x = r.x + offsetH;
		int y = r.y + offsetV;

		Point p = new PrecisionPoint(x, y);
		getOwner().translateToAbsolute(p);
		return p;
	}

	/**
	 * @see org.eclipse.draw2d.AbstractConnectionAnchor#getReferencePoint()
	 */
	public Point getReferencePoint() {
		return getLocation(null);
	}

	/**
	 * @param offsetH
	 *            The offsetH to set.
	 */
	public void setOffsetH(int offsetH) {
		this.offsetH = offsetH;
		fireAnchorMoved();
	}

	/**
	 * @param offsetV
	 *            The offsetV to set.
	 */
	public void setOffsetV(int offsetV) {
		this.offsetV = offsetV;
		fireAnchorMoved();
	}

	/**
	 * @return identifier
	 */
	public int getId() {
		return id;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o) {
		if (o instanceof NodeConnectionAnchor) {
			NodeConnectionAnchor fa = (NodeConnectionAnchor) o;

			if (fa.offsetH == this.offsetH && fa.offsetV == this.offsetV
					&& fa.getOwner() == this.getOwner()) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return (this.offsetH * 43 + this.offsetV * 47)
				^ this.getOwner().hashCode();
	}

}
