package roneditor.objectnetview;

import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.jface.resource.ImageDescriptor;

import roneditor.objectnetview.model.ObjectNetModelCreationFactory;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * Main class for the ObjectNet palette. All graphical figures have to be
 * registered here.
 */
public class ObjectNetPaletteRoot extends PaletteRoot {
	protected PaletteGroup controls = new PaletteGroup("Controls");

	protected PaletteGroup symbols = new PaletteGroup("Symbols");

	/**
	 * Creates a new ObjectNetPaletteRoot instance.
	 * 
	 */
	public ObjectNetPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols

		add(symbols);

		CreationToolEntry entry;
		ImageDescriptor iconPlace;
		iconPlace = ImageDescriptor.createFromFile(ObjectNetPaletteRoot.class,
				"/icons/place.png");
		ImageDescriptor iconTransition;
		iconTransition = ImageDescriptor.createFromFile(
				ObjectNetPaletteRoot.class, "/icons/transition.png");
		ImageDescriptor iconArc;
		iconArc = ImageDescriptor.createFromFile(ObjectNetPaletteRoot.class,
				"/icons/arc.png");
		entry = new ObjectNetCreationToolEntry("Place", "Creates a ONPlace",
				new ObjectNetModelCreationFactory(ONPlace.class), iconPlace,
				null);
		symbols.add(entry);

		entry = new ObjectNetCreationToolEntry("Transition",
				"Creates a ONTransition", new ObjectNetModelCreationFactory(
						ONTransition.class), iconTransition, null);
		symbols.add(entry);

		entry = new ObjectNetConnectionCreationToolEntry("Arc",
				"Creates a ONArc", new ObjectNetModelCreationFactory(
						ONArc.class), iconArc, null);
		symbols.add(entry);
	}
}
