package roneditor.objectnetview.actions;

/**
 * Edit action for the properties dialog,
 * where the user can change the place or transition properties.
 */
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.RONActivator;
import roneditor.objectnetview.commands.EditONArcCommand;
import roneditor.objectnetview.commands.EditONPlaceCommand;
import roneditor.objectnetview.commands.EditONTransitionCommand;
import roneditor.objectnetview.dialogs.ONArcPropertiesDialog;
import roneditor.objectnetview.dialogs.ONPlacePropertiesDialog;
import roneditor.objectnetview.dialogs.ONTransitionPropertiesDialog;
import roneditor.objectnetview.edit.parts.ONArcEditPart;
import roneditor.objectnetview.edit.parts.ONPlaceEditPart;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

public class EditAction extends SelectionAction {

	static public String ID = "Properties_Dialog_Action";

	public EditAction(IWorkbenchPart part) {
		super(part);
		setDescription("Properties Dialog Action");
		setText("Properties Dialog");
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute(getCommand());
	}

	/**
	 * @return the command class from the place or transition edit part.
	 */
	public Command getCommand() {
		Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();

		if (selection instanceof ONPlaceEditPart) {
			return getEditONPlaceCommand((ONPlaceEditPart) selection);
		} else if (selection instanceof ONTransitionEditPart) {
			return getEditONTransitionCommand((ONTransitionEditPart) selection);
		} else if (selection instanceof ONArcEditPart) {
			return getEditONArcCommand((ONArcEditPart) selection);
		}
		return null;
	}
	/**
	 * ffnet ein {@link ONPlacePropertiesDialog} fr Benutzereingaben und 
	 * liefert ein {@link EditONPlaceCommand} zurck
	 * @param editPart {@link ONPlaceEditPart editPart} for a {@link ONPlace}
	 * @return {@link EditONPlaceCommand}
	 */
	public EditONPlaceCommand getEditONPlaceCommand(ONPlaceEditPart editPart) {
		EditONPlaceCommand editCommand = new EditONPlaceCommand();
		ONPlace onPlace = (ONPlace) editPart.getModel();
		editCommand.setONNode(onPlace);
		Shell shell = RONActivator.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
		ONPlacePropertiesDialog d = new ONPlacePropertiesDialog(shell, SWT.NULL);
		d.setTitle("Edit ONPlace");
		d.setPlaceName(onPlace.getName());
		d.setTokens(onPlace.getTokens());
		d.open();
		String placeName = d.getPlaceName() != null
				&& d.getPlaceName().equals("") ? null : d.getPlaceName();
		if (d.isCancelled() || d.getTokens() < 0) {
			return null;
		}
		if (placeName != null && placeName.equals(onPlace.getName())
				&& d.getTokens() == onPlace.getTokens()) {
			return null;
		}
		editCommand.setNewName(placeName);
		editCommand.setNewTokens(d.getTokens());
		return editCommand;
	}
	/**
	 * ffnet ein {@link ONTransitionPropertiesDialog} fr Benutzereingaben und 
	 * liefert ein {@link EditONTransitionCommand} zurck
	 * @param editPart {@link ONTransitionEditPart editPart} for a {@link ONTransition}
	 * @return {@link EditONTransitionCommand}
	 */
	 
	public EditONTransitionCommand getEditONTransitionCommand(
			ONTransitionEditPart editPart) {
		EditONTransitionCommand editCommand = new EditONTransitionCommand();
		ONTransition onTransition = (ONTransition) editPart.getModel();
		editCommand.setONNode(onTransition);
		Shell shell = RONActivator.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
		ONTransitionPropertiesDialog d = new ONTransitionPropertiesDialog(
				shell, SWT.NULL);
		d.setTitle("Edit ONTransition");
		d.setTransitionName(onTransition.getName());
		d.open();
		String transitionName = d.getTransitionName() != null
				&& d.getTransitionName().equals("") ? null : d
				.getTransitionName();
		if (d.isCancelled()) {
			return null;
		}
		if (transitionName != null
				&& transitionName.equals(onTransition.getName())) {
			return null;
		}
		editCommand.setNewName(transitionName);
		return editCommand;
	}

	/**
	 * ffnet ein {@link ONArcPropertiesDialog} fr Benutzereingaben und
	 * liefert ein {@link EditONArcCommand} zurck
	 * @param editPart {@link ONArcEditPart editPart} for a {@link ONArc}
	 * @return {@link EditONArcCommand}
	 */
	public EditONArcCommand getEditONArcCommand(ONArcEditPart editPart) {
		EditONArcCommand editCommand = new EditONArcCommand();
		ONArc onArc = (ONArc) editPart.getModel();
		editCommand.setOnArc(onArc);
		Shell shell = RONActivator.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
		ONArcPropertiesDialog d = new ONArcPropertiesDialog(shell, SWT.NULL);
		d.setTitle("Edit ONArc");
		d.setWeight(onArc.getWeight());
		d.open();
		if (d.isCancelled() || d.getWeight() <= 0) {
			return null;
		}
		if (d.getWeight() == onArc.getWeight()) {
			return null;
		}
		editCommand.setNewWeight(d.getWeight());
		return editCommand;
	}

	/**
	 * @return true if the action could be executed, false otherwise.
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * @return true if the action could be executed, false otherwise.
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if (part.getModel() instanceof ONNode
					|| part.getModel() instanceof ONArc)
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
