package roneditor.objectnetview.actions;

import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.objectnetview.commands.FireONTransitionCommand;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import roneditor.objectnetview.edit.parts.ObjectNetEditPart;
import roneditor.objectnetview.model.ObjectNetModelUtil;
import ronmodel.ONTransition;
/**
 * Action Fr feuern einem {@link ONTransition}.
 *
 */
public class FireONTransitionAction extends SelectionAction {

	static public String ID = "Fire_ONTransition_Action";

	public FireONTransitionAction(IWorkbenchPart part) {
		super(part);
		setDescription("Fire ONTransition Action");
		setText("Fire");
	}

	/**
	 * Von jeder Vorbereichsstelle dieser Transition wird soviel Tokens entfernt 
	 * wie der Gewicht der zugehrige Eingangskante und in jeder Nachbereichstelle 
	 * wird soviel tokens eingefgt wie der Gewicht der zugehrige Ausgangskante
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute(getCommand());
	}

	/**
	 * @return {@link FireONTransitionCommand}
	 */
	public Command getCommand() {
		Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();
		if (selection instanceof ONTransitionEditPart) {
			FireONTransitionCommand command = new FireONTransitionCommand();
			command.setOnTransition(((ONTransitionEditPart) selection)
					.getONTransition());
			return command;
		}
		if (selection instanceof ObjectNetEditPart) {
			ObjectNetEditPart editPart = (ObjectNetEditPart) selection;
			Vector<ONTransition> activeTransitions = ObjectNetModelUtil
					.getActiveONTransitions(editPart.getObjectNet());
			int random = (int) (Math.random() * activeTransitions.size());
			FireONTransitionCommand command = new FireONTransitionCommand();
			command.setOnTransition(activeTransitions.get(random));
			return command;
		}
		return null;
	}

	/**
	 * @return true wenn in jeder Vorbereichsstelle dieser Transition 
	 * mindestens soviel Tokens gibt als die Gewicht der zugehrige Kante 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * @return true wenn in jeder Vorbereichsstelle dieser Transition 
	 * mindestens soviel Tokens gibt als die Gewicht der zugehrige Kante 
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().size() != 1)
			return false;
		List parts = getSelectedObjects();
		Object o = parts.get(0);
		if (o instanceof ObjectNetEditPart) {
			ObjectNetEditPart editPart = (ObjectNetEditPart) o;
			Vector<ONTransition> activeTransitions = ObjectNetModelUtil
					.getActiveONTransitions(editPart.getObjectNet());
			return !activeTransitions.isEmpty();
		}
		if (o instanceof ONTransitionEditPart) {
			ONTransition onTransition = ((ONTransitionEditPart) o)
					.getONTransition();
			return ObjectNetModelUtil.isActive(onTransition);
		}
		return false;
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
