package roneditor.objectnetview.commands;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command for deleting a ONArc.
 */
public class DeleteONArcCommand extends org.eclipse.gef.commands.Command {
	private static final String DeleteCommand_Label = "DeleteONArcCommand";

	private ONArc onArc;

	private ONNode source;

	private ONNode target;

	private ObjectNet objectNet;

	/**
	 * Constructor
	 */
	public DeleteONArcCommand() {
		super(DeleteCommand_Label);
		setLabel("delete Arc");
	}

	/**
	 * Entry point for command execution.
	 * ein {@link ONArc} wird aus einem {@link ObjectNet} gelscht
	 */
	public void execute() {
		source = onArc.getSource();
		target = onArc.getTarget();
		objectNet = onArc.getObjectNet();
		onArc.setSource(null);
		onArc.setTarget(null);
		onArc.setObjectNet(null);
	}

	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		onArc.setSource(source);
		onArc.setTarget(target);
		onArc.setObjectNet(objectNet);
	}

	/**
	 * @return the objectNet
	 */
	public ObjectNet getObjectNet() {
		return objectNet;
	}

	/**
	 * @param objectNet
	 *            the objectNet to set
	 */
	public void setObjectNet(ObjectNet objectNet) {
		this.objectNet = objectNet;
	}

	/**
	 * @return the onArc
	 */
	public ONArc getONArc() {
		return onArc;
	}

	/**
	 * @param onArc
	 *            the onArc to set
	 */
	public void setONArc(ONArc onArc) {
		this.onArc = onArc;
	}

	/**
	 * @return the source
	 */
	public ONNode getSource() {
		return source;
	}

	/**
	 * @param source
	 *            the source to set
	 */
	public void setSource(ONNode source) {
		this.source = source;
	}

	/**
	 * @return the target
	 */
	public ONNode getTarget() {
		return target;
	}

	/**
	 * @param target
	 *            the target to set
	 */
	public void setTarget(ONNode target) {
		this.target = target;
	}

	@Override
	public boolean canExecute() {
		return onArc != null && onArc.getSource() != null
				&& onArc.getTarget() != null && onArc.getObjectNet() != null;
	}

}
