package roneditor.objectnetview.edit.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.edit.policies.ArcSelectionHandlesEditPolicy;
import roneditor.objectnetview.edit.policies.ONArcBendPointEditPolicy;
import roneditor.objectnetview.edit.policies.ONArcEditPolicy;
import roneditor.objectnetview.edit.policies.ONArcEndpointEditPolicy;
import roneditor.objectnetview.figures.ONArcFigure;
import roneditor.objectnetview.model.ONArcPropertySource;
import ronmodel.BendPoint;
import ronmodel.ONArc;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a {@link ONArc}.
 */
public class ONArcEditPart extends AbstractConnectionEditPart {

	private Notifier target;

	private ONArcAdapter adapter;

	private IPropertySource propertySource = null;

	public ONArcEditPart(ONArc onArc) {
		super();
		setModel(onArc);
	}
	/**
	 * 
	 * @return the {@link ONArc onArc}
	 */
	public ONArc getONArc() {
		return (ONArc) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new ONArcEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ArcSelectionHandlesEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new ONArcBendPointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new ONArcEditPolicy());
	}
	/** 
	 * create a {@link ONArcFigure figure} for the {@link ONArc onArc}
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		Connection c = (Connection) super.createFigure();
		return new ONArcFigure(c);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new ONArcAdapter();
		getONArc().eAdapters().add(adapter);
		super.activate();
	}

	/**
	 * {@link ONArcFigure figure} aktualisieren wenn das Modell
	 * gendert wird
	 */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		ONArcFigure onArcFigure = (ONArcFigure) getConnectionFigure();
		onArcFigure.setWeight(getONArc().getWeight());
		refreshBendpoints();
	}

	/**
	 * Hilfsfunktion fuer {@link #refreshVisuals()}.
	 * hier werden die Positionen von {@link BendPoint bendpoints} 
	 * aktualisiert
	 * 
	 */
	private void refreshBendpoints() {
			List modelConstraint = getONArc().getBendPoints();
			List figureConstraint = new ArrayList();
			if (modelConstraint != null) {
				for (int i = 0; i < modelConstraint.size(); i++) {
					BendPoint wbp = (BendPoint) modelConstraint.get(i);
					RelativeBendpoint rbp =
						new RelativeBendpoint(getConnectionFigure());
					Dimension first =
						new Dimension((Integer) wbp.getW().get(0), (Integer) wbp.getH().get(0));
					Dimension second;
					if(wbp.getW().size()==1){
						second = new Dimension((Integer) wbp.getW().get(0), (Integer) wbp.getH().get(1));
					}
					else{
						second = new Dimension((Integer) wbp.getW().get(1), (Integer) wbp.getH().get(1));
					}
					rbp.setRelativeDimensions(first, second);
					figureConstraint.add(rbp);
				}
				getConnectionFigure().setRoutingConstraint(figureConstraint);
			}		
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new ONArcPropertySource(getONArc());
		}
		return propertySource;
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class ONArcAdapter implements Adapter {
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.ON_ARC__WEIGHT:
				refreshVisuals();
				break;
			case RonmodelPackage.ON_ARC__BEND_POINTS:
				refreshBendpoints();
				break;
			}
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(type.getClass());
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class
}
