package roneditor.objectnetview.edit.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.objectnetview.commands.CreateONArcCommand;
import roneditor.objectnetview.commands.ReconnectONSourceCommand;
import roneditor.objectnetview.commands.ReconnectONTargetCommand;
import roneditor.objectnetview.edit.parts.AbstractONNodeEditPart;
import ronmodel.ONArc;
import ronmodel.ONNode;

/**
 * EditPolicy for a ONNode.
 */
public class ONNodeEditPolicy extends GraphicalNodeEditPolicy {

	public ONNodeEditPolicy() {
		super();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 * @return {@link CreateONArcCommand}
	 */
	protected Command getConnectionCompleteCommand(
			CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		AbstractONNodeEditPart sourceEditPart = (AbstractONNodeEditPart) request
				.getSourceEditPart();
		AbstractONNodeEditPart targetEditPart = (AbstractONNodeEditPart) request
				.getTargetEditPart();
		if (command instanceof CreateONArcCommand) {
			CreateONArcCommand commandONNode = (CreateONArcCommand) command;
			commandONNode.setTarget(getONNode());
			commandONNode.setSourceAnchorID(sourceEditPart.getAnchorID());
			commandONNode.setTargetAnchorID(targetEditPart.getAnchorID());
		}
		return command;
	}

	protected ONNode getONNode() {
		return (ONNode) getHost().getModel();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 * @return {@link CreateONArcCommand}
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		CreateONArcCommand command = new CreateONArcCommand();
		Object reqObj = request.getNewObject();
		if (reqObj instanceof ONArc) {
			command.setONArc((ONArc) reqObj);
		}
		command.setSource(getONNode());
		request.setStartCommand(command);
		return command;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 * @return {@link ReconnectONSourceCommand}
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		Object reqObj = request.getConnectionEditPart().getModel();
		ReconnectONSourceCommand command = null;
		if (reqObj instanceof ONArc) {
			command = new ReconnectONSourceCommand();
			command.setONArc((ONArc) reqObj);
			command.setNewSource((ONNode) request.getTarget().getModel());
		}
		return command;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 * @return {@link ReconnectONTargetCommand}
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		Object reqObj = request.getConnectionEditPart().getModel();
		ReconnectONTargetCommand command = null;
		if (reqObj instanceof ONArc) {
			command = new ReconnectONTargetCommand();
			command.setONArc((ONArc) reqObj);
			command.setNewTarget((ONNode) request.getTarget().getModel());
		}
		return command;
	}
}
