package roneditor.objectnetview.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import roneditor.figures.AbstractNodeFigure;
import roneditor.figures.NodeConnectionAnchor;
import ronmodel.ONPlace;

/**
 * Represents the graphical layout of a {@link ONPlace}.
 */
public class ONPlaceFigure extends AbstractNodeFigure {
	public static final int NUMBER_OF_ANCHORS = 4;

	protected Label label;

	protected XYLayout layout;

	private TokenEllipse tokenEllipse;

	public ONPlaceFigure() {
		super();
		layout = new XYLayout();
		setLayoutManager(layout);
		setOutline(false);
		// setOpaque(false);
		tokenEllipse = new TokenEllipse();
		tokenEllipse.setBackgroundColor(ColorConstants.lightGray);
		// createConnectionAnchors(NUMBER_OF_ANCHORS,tokenEllipse);
		connectionAnchors.put(0, new EllipseAnchor(tokenEllipse));
		label = new Label("  ");
		add(tokenEllipse);
		add(label);
	}

	public void setName(String name) {
		if (name == null) {
			label.setText("  ");
		} else {
			label.setText(" " + name + " ");
		}
		validate();
	}

	public void setTokens(int tokens) {
		tokenEllipse.setTokens(tokens);
		validate();
	}

	public void setMappingID(int mappingID) {
		if (mappingID > 0 && showingMappingID) {
			label.setText(mappingID + ":" + label.getText());
			validate();
		}
	}

	public void validate() {
		layout.setConstraint(tokenEllipse,
				new Rectangle(
						(getSize().width - tokenEllipse.getSize().width) / 2,
						1, -1, -1));
		layout.setConstraint(label, new Rectangle((getSize().width - label
				.getSize().width) / 2, tokenEllipse.getSize().height + 5, -1,
				-1));
		// refreshConnectionAnchors();
		super.validate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return getPreferredSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = Math.max(tokenEllipse.getSize().width,
				label.getSize().width);
		int height = tokenEllipse.getSize().height + label.getSize().height + 5;
		return new Dimension(width, height);
	}

	/**
	 * bei nderungen werden die ConnectionAnchors aktualisiert
	 */
	private void refreshConnectionAnchors() {
		NodeConnectionAnchor anchor = null;
		int w = getBounds().width / 2;
		int h = getBounds().height / 2;

		anchor = (NodeConnectionAnchor) connectionAnchors.get(1);
		anchor.setOffsetH(w);
		anchor.setOffsetV(0);

		anchor = (NodeConnectionAnchor) connectionAnchors.get(2);
		anchor.setOffsetH(getBounds().width - 1);
		anchor.setOffsetV(h);

		anchor = (NodeConnectionAnchor) connectionAnchors.get(3);
		anchor.setOffsetH(w);
		anchor.setOffsetV(getBounds().height - 1);

		anchor = (NodeConnectionAnchor) connectionAnchors.get(4);
		anchor.setOffsetH(0);
		anchor.setOffsetV(h);
	}

	private static class TokenEllipse extends Ellipse {
		Label label;

		XYLayout layout;

		public TokenEllipse() {
			layout = new XYLayout();
			setLayoutManager(layout);
			setOpaque(false);
			setBackgroundColor(ColorConstants.white);
			label = new Label(" ");
			add(label);
		}

		public void setTokens(int tokens) {
			if (tokens == 0)
				label.setText("   ");
			else
				label.setText(" " + tokens + " ");

			validate();
		}

		@Override
		protected boolean useLocalCoordinates() {
			return true;
		}

		@Override
		public void validate() {
			layout.setConstraint(label, new Rectangle(5,
					(getSize().height - label.getSize().height) / 2, -1, -1));
			super.validate();
		}

		@Override
		public Dimension getPreferredSize(int arg0, int arg1) {
			int diam = Math.max(label.getSize().width, label.getSize().height) + 10;
			return new Dimension(diam, diam);
		}

		@Override
		public Dimension getMinimumSize(int arg0, int arg1) {
			return getPreferredSize();
		}
	}
}