package roneditor.objectnetview.model;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import roneditor.model.NumberCellEditorValidator;
import ronmodel.ONPlace;

/**
 * Property source for onPlace.
 */
public class ONPlacePropertySource implements IPropertySource {
	final public static int ID_NAME = 0;

	final public static int ID_TOKENS = 1;

	final public static int ID_X = 2;

	final public static int ID_Y = 3;

	final private static IPropertyDescriptor[] descriptors;

	static {
		final PropertyDescriptor nameProp = new PropertyDescriptor(Integer
				.valueOf(ID_NAME), "name");
		final PropertyDescriptor tokensProp = new PropertyDescriptor(Integer
				.valueOf(ID_TOKENS), "tokens");
		tokensProp.setValidator(NumberCellEditorValidator.instance(true));
		final PropertyDescriptor xProp = new PropertyDescriptor(Integer
				.valueOf(ID_X), "x");
		xProp.setValidator(NumberCellEditorValidator.instance(false));
		final PropertyDescriptor yProp = new PropertyDescriptor(Integer
				.valueOf(ID_Y), "y");
		yProp.setValidator(NumberCellEditorValidator.instance(false));
		descriptors = new IPropertyDescriptor[] { nameProp, tokensProp, xProp,
				yProp };
	}

	private ONPlace onPlace;

	public ONPlacePropertySource(final ONPlace onPlace) {
		this.onPlace = onPlace;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(final Object id) {
		switch (((Integer) id).intValue()) {
		case ID_NAME:
			return onPlace.getName();
		case ID_TOKENS:
			return Integer.valueOf(onPlace.getTokens()).toString();
		case ID_X:
			return Integer.valueOf(onPlace.getX()).toString();
		case ID_Y:
			return Integer.valueOf(onPlace.getY()).toString();
		default:
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	public void setPropertyValue(final Object id, final Object value) {
		switch (((Integer) id).intValue()) {
		case ID_NAME:
			onPlace.setName((String) value);
			break;
		case ID_TOKENS:
			onPlace.setTokens(Integer.valueOf((String) value));
			break;
		case ID_X:
			onPlace.setX(Integer.valueOf((String) value));
			break;
		case ID_Y:
			onPlace.setY(Integer.valueOf((String) value));
			break;
		default:
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return onPlace;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return descriptors;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(final Object propName) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(final Object propName) {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "ONPlace(" + onPlace.getName() + "," + onPlace.getTokens() + ","
				+ onPlace.getX() + "," + onPlace.getY() + ")";
	}

}
