package roneditor.objectnetview.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;

/**
 * This class implements the CreationFactory used by the CreationTool. It in
 * turn uses the EMF-generated factories to create the model instances
 * 
 */
public class ObjectNetModelCreationFactory implements CreationFactory {
	private Class targetClass;

	public ObjectNetModelCreationFactory(Class targetClass) {
		this.targetClass = targetClass;
	}

	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String ronmodelURI = RonmodelPackage.eNS_URI;
		RonmodelPackage ronmodelPackage = (RonmodelPackage) registry
				.get(ronmodelURI);
		RonmodelFactory factory = ronmodelPackage.getRonmodelFactory();

		Object result = null;
		if (targetClass.equals(ONPlace.class)) {
			result = factory.createONPlace();
		} else if (targetClass.equals(ONTransition.class)) {
			result = factory.createONTransition();
		} else if (targetClass.equals(ONArc.class)) {
			result = factory.createONArc();
		}
		return result;
	}

	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}
}
