package roneditor.ronview;

import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.jface.resource.ImageDescriptor;
import roneditor.ronview.model.RONModelCreationFactory;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;

/**
 * Main class for the ronview palette. All graphical figures have to be
 * registered here.
 */
public class RONPaletteRoot extends PaletteRoot {
	protected PaletteGroup controls = new PaletteGroup("Controls");

	protected PaletteGroup symbols = new PaletteGroup("Symbols");

	/**
	 * Creates a new NetworkPaletteRoot instance.
	 * 
	 */
	public RONPaletteRoot() {
		// create root
		super();

		// a group of default control tools
		add(controls);

		// the selection tool
		ToolEntry tool = new SelectionToolEntry();
		controls.add(tool);

		// use selection tool as default entry
		setDefaultEntry(tool);

		// the marquee selection tool
		controls.add(new MarqueeToolEntry());

		// a group for the editor symbols

		add(symbols);

		CreationToolEntry entry;
		ImageDescriptor iconPlace;
		iconPlace = ImageDescriptor.createFromFile(RONPaletteRoot.class,
				"/icons/RONPlace.png");
		ImageDescriptor iconTransition;
		iconTransition = ImageDescriptor.createFromFile(RONPaletteRoot.class,
				"/icons/RONTransition2.png");
		ImageDescriptor iconArc;
		iconArc = ImageDescriptor.createFromFile(RONPaletteRoot.class,
				"/icons/arc.png");
		entry = new RONCreationToolEntry("Place", "Creates a HLPlace",
				new RONModelCreationFactory(HLPlace.class), iconPlace, null);
		symbols.add(entry);

		entry = new RONCreationToolEntry("Transition",
				"Creates a HLTransition", new RONModelCreationFactory(
						HLTransition.class), iconTransition, null);
		symbols.add(entry);

		entry = new ConnectionCreationToolEntry("Arc", "Creates a HLArc",
				new RONModelCreationFactory(HLArc.class), iconArc, null);
		symbols.add(entry);
	}
}
