package roneditor.ronview.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.converter.RuleSimulator;
import roneditor.dialogs.FireDialog;
import roneditor.dialogs.MatchDialog;
import roneditor.objectnetview.model.ObjectNetModelUtil;
import roneditor.ronview.commands.FireApplyRuleHLTransitionCommand;
import roneditor.ronview.commands.FireFireHLTransitionCommand;
import roneditor.ronview.commands.FireSplitHLTransitionCommand;
import roneditor.ronview.commands.FireStandardHLTransitionCommand;
import roneditor.ronview.edit.parts.HLTransitionEditPart;
import roneditor.ronview.model.RONModelUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.NAC;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

public class FireHLTransitionAction extends SelectionAction {

	static public String ID = "Fire_HLTransition_Action";

	public FireHLTransitionAction(IWorkbenchPart part) {
		super(part);
		setDescription("Fire HLTransition Action");
		setText("Fire");
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		Command command = getCommand();
		if (command != null) {
			execute(command);
		}
	}

	/**
	 * @return {@link FireApplyRuleHLTransitionCommand} or
	 * {@link FireFireHLTransitionCommand} or
	 * {@link FireSplitHLTransitionCommand} or 
	 * {@link FireStandardHLTransitionCommand}
	 */
	public Command getCommand() {
		Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();
		if (selection instanceof HLTransitionEditPart) {
			HLTransitionEditPart part = (HLTransitionEditPart) selection;
			HLTransition transition = part.getHLTransition();
			if (transition.getKind() == HLTransitionKind.STANDARD_LITERAL) {
				FireStandardHLTransitionCommand command = new FireStandardHLTransitionCommand();
				command.setTransition(transition);
				return command;
			}
			if (transition.getKind() == HLTransitionKind.FIRE_LITERAL) {
				Collection<HLPlace> sourceNetPlaces = RONModelUtil
						.getSourceNetPlaces(transition);
				HLPlace sourcePlace = sourceNetPlaces.iterator().next();
				ObjectNet sourceObjectNet = (ObjectNet) RONModelUtil
						.getSelectedTokens(sourcePlace).iterator().next();
				ObjectNet copyOfSourceON = (ObjectNet) EcoreUtil
						.copy(sourceObjectNet);
				List<ObjectNet> selectedNets = new Vector<ObjectNet>();
				selectedNets.add(copyOfSourceON);
				FireDialog fireDialog = new FireDialog(selectedNets);
				fireDialog.open();
				ONTransition onTransition = fireDialog.getTransitionToFire();
				if (onTransition == null) {
					return null;
				}
				FireFireHLTransitionCommand command = new FireFireHLTransitionCommand();
				command.setHLTransition(transition);
				command.setONTransition(onTransition);
				command.setSourceObjectNet(sourceObjectNet);
				command.setCopyOfSourceObjectNet(copyOfSourceON);
				return command;
			}
			
			
			if (transition.getKind() == HLTransitionKind.APPLYRULE_LITERAL) {
				FireApplyRuleHLTransitionCommand command = new FireApplyRuleHLTransitionCommand();
				Collection<Token> sourceObjectNets = new HashSet<Token>();
				Rule sourceRule = null;

				for (Object o : transition.getIncoming()) {
					HLPlace sourcePlace = (HLPlace) ((HLArc) o).getSource();
					Token selectedToken = RONModelUtil.getSelectedTokens(sourcePlace)
							.iterator().next();
					command.getSourceTokens().put(selectedToken, sourcePlace);
					if (sourcePlace.getType() == HLPlaceType.NET_LITERAL) {
						ObjectNet objectNet = (ObjectNet) selectedToken;
						sourceObjectNets.add(objectNet);
					} else {
						sourceRule = (Rule) selectedToken;
					}
				}

				ObjectNet unionObjectNet = RONModelUtil.union(sourceObjectNets);

				// delete Empty NACs
				Rule copyOfSourceRule = (Rule) EcoreUtil.copy(sourceRule);
				Collection<NAC> emptyNacs = new Vector<NAC>();
				for (Object o : copyOfSourceRule.getNacs()) {
					NAC nac = (NAC) o;
					if (nac.getMappings().isEmpty()) {
						emptyNacs.add(nac);
					}
				}
				for (NAC nac : emptyNacs) {
					nac.setRule(null);
				}
				
				 ArrayList<Rule> rules=new ArrayList<Rule>();
				 rules.add(copyOfSourceRule); 
				 ArrayList<ObjectNet> objectNets=new ArrayList<ObjectNet>(); 
				 objectNets.add(unionObjectNet); 
				 MatchDialog matchDialog=new MatchDialog(rules,objectNets); 
				 matchDialog.open();
				 if (matchDialog.isCanceled()) {
					return null;
				}
				unionObjectNet=matchDialog.getObjectNetCopy(); 
				command.setTransition(transition);
				command.setUnionObjectNet(unionObjectNet);
				command.setSourceRule(sourceRule);
				command.setSourceRuleNoEmptyNac(copyOfSourceRule);
				command.setMatchToApply(matchDialog.getMatchToApply());
				return command;
			}
			if (transition.getKind() == HLTransitionKind.SPLIT_LITERAL) {
				FireSplitHLTransitionCommand command = new FireSplitHLTransitionCommand();
				command.setTransition(transition);
				return command;
			}
		}
		return null;
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * @return true if the action could be executed, false otherwise.
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().size() != 1)
			return false;
		List parts = getSelectedObjects();
		Object o = parts.get(0);
		if (!(o instanceof HLTransitionEditPart))
			return false;
		HLTransition transition = ((HLTransitionEditPart) o).getHLTransition();
		if (transition.getKind() == HLTransitionKind.STANDARD_LITERAL) {
			return checkStandard(transition);
		} else if (transition.getKind() == HLTransitionKind.FIRE_LITERAL) {
			return checkFire(transition);
		} else if (transition.getKind() == HLTransitionKind.APPLYRULE_LITERAL) {
			return checkApplyRule(transition);
		} else if (transition.getKind() == HLTransitionKind.SPLIT_LITERAL) {
			return checkSplit(transition);
		}
		return false;
	}

	/**
	 *  @return true if the action for a {@link HLTransition StandardTransition} could be executed, false otherwise.
	 */
	private boolean checkStandard(HLTransition transition) {
		HLArc input = null;
		HLArc output = null;
		HLPlace source = null;
		HLPlace target = null;
		int nbOfSourceRulePlaces = 0;
		int nbOfSourceNetPlaces = 0;
		for (Object o : transition.getIncoming()) {
			input = (HLArc) o;
			source = (HLPlace) input.getSource();
			if (RONModelUtil.getSelectedTokens(source).size() != 1) {
				return false;
			}
			if (source.getType() == HLPlaceType.RULE_LITERAL) {
				if (++nbOfSourceRulePlaces > 1) {
					return false;
				}
			} else {
				++nbOfSourceNetPlaces;
			}
		}
		if (nbOfSourceNetPlaces == 0 && nbOfSourceRulePlaces == 0) {
			return false;
		}

		// Nachbereich
		for (Object o : transition.getOutgoing()) {
			output = (HLArc) o;
			target = (HLPlace) output.getTarget();
			if (target.getType() == HLPlaceType.RULE_LITERAL) {
				if (nbOfSourceRulePlaces == 0) {
					return false;
				}
			} else {
				if (nbOfSourceNetPlaces == 0) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 *  @return true if the action for a {@link HLTransition FireTransition} could be executed, false otherwise.
	 */
	private boolean checkFire(HLTransition transition) {
		HLArc input = null;
		HLArc output = null;
		HLPlace source = null;
		HLPlace target = null;
		HLPlace netPlace = null;
		int nbOfSourceNetPlaces = 0;
		int nbOfTargetNetPlaces = 0;
		for (Object o : transition.getIncoming()) {
			input = (HLArc) o;
			source = (HLPlace) input.getSource();
			if (source.getType() == HLPlaceType.RULE_LITERAL) {
				return false;
			}
			if (source.getType() == HLPlaceType.NET_LITERAL) {
				if (++nbOfSourceNetPlaces > 1) {
					return false;
				}
				netPlace = source;
			}
		}
		if (netPlace == null
				|| RONModelUtil.getSelectedTokens(netPlace).size() != 1) {
			return false;
		}
		for (Object o : transition.getOutgoing()) {
			output = (HLArc) o;
			target = (HLPlace) output.getTarget();
			if (target.getType() == HLPlaceType.RULE_LITERAL) {
				return false;
			}
			if (target.getType() == HLPlaceType.NET_LITERAL) {
				++nbOfTargetNetPlaces;
			}
		}
		if (nbOfTargetNetPlaces == 0) {
			return false;
		}
		ObjectNet objectNet = (ObjectNet) RONModelUtil.getSelectedTokens(
				netPlace).iterator().next();
		return !ObjectNetModelUtil.getActiveONTransitions(objectNet).isEmpty();
	}

	/**
	 *  @return true if the action for a {@link HLTransition ApplyRuleTransition} could be executed, false otherwise.
	 */
	private boolean checkApplyRule(HLTransition transition) {
		Collection<HLPlace> sourceNetPlaces = new HashSet<HLPlace>();
		HLPlace sourceRulePlace = null;

		for (Object o : transition.getIncoming()) {
			HLPlace sourcePlace = (HLPlace) ((HLArc) o).getSource();
			if (sourcePlace.getType() == HLPlaceType.NET_LITERAL) {
				sourceNetPlaces.add(sourcePlace);
			} else {
				if (sourceRulePlace != null) {
					return false;
				} else {
					sourceRulePlace = sourcePlace;
				}
			}
		}
		if (sourceRulePlace == null || sourceNetPlaces.isEmpty()) {
			return false;
		}

		Collection<Token> selectedObjectNets = new HashSet<Token>();
		Rule selectedRule = null;

		for (HLPlace sourceNetPlace : sourceNetPlaces) {
			Collection<Token> selectedTokens = RONModelUtil
					.getSelectedTokens(sourceNetPlace);
			if (selectedTokens.size() != 1) {
				return false;
			}
			selectedObjectNets
					.add((ObjectNet) selectedTokens.iterator().next());
		}
		Collection<Token> selectedTokens = RONModelUtil
				.getSelectedTokens(sourceRulePlace);
		if (selectedTokens.size() != 1) {
			return false;
		}
		selectedRule = (Rule) selectedTokens.iterator().next();
		ObjectNet unionObjectNet = RONModelUtil.union(selectedObjectNets);

		Rule copyOfSourceRule = (Rule) EcoreUtil.copy(selectedRule);
		Collection nacs = new Vector();
		nacs.addAll(copyOfSourceRule.getNacs());
		for (Object o : nacs) {
			NAC nac = (NAC) o;
			if (nac.getMappings().isEmpty()) {
				nac.setRule(null);
			}
		}

		if (!new RuleSimulator().isApplicable(copyOfSourceRule, unionObjectNet)) {
			return false;
		} else
			return true;
	}

	/**
	 *  @return true if the action for a {@link HLTransition SplitTransition} could be executed, false otherwise.
	 */
	private boolean checkSplit(HLTransition transition) {
		HLArc input = null;
		HLArc output = null;
		HLPlace source = null;
		HLPlace target = null;

		int nbOfSourceNetPlaces = 0;
		int nbOfTargetNetPlaces = 0;
		for (Object o : transition.getIncoming()) {
			input = (HLArc) o;
			source = (HLPlace) input.getSource();
			if (RONModelUtil.getSelectedTokens(source).size() != 1) {
				return false;
			}
			if (source.getType() == HLPlaceType.RULE_LITERAL) {
				return false;
			}
			if (++nbOfSourceNetPlaces > 1) {
				return false;
			}
		}
		if (nbOfSourceNetPlaces == 0) {
			return false;
		}
		for (Object o : transition.getOutgoing()) {
			output = (HLArc) o;
			target = (HLPlace) output.getTarget();
			if (target.getType() == HLPlaceType.RULE_LITERAL) {
				return false;
			}
			if (++nbOfTargetNetPlaces > 1) {
				return false;
			}
		}
		if (nbOfTargetNetPlaces == 0) {
			return false;
		}
		return true;
	}

	/** 
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}
