package roneditor.ronview.commands;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;

import roneditor.ronview.model.RONModelUtil;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ObjectNet;
import ronmodel.Token;
/**
 * command zum feuern eines {@link HLTransition splitHLTransition}
 */
public class FireSplitHLTransitionCommand extends Command {
	private HLTransition transition;

	private Map<Token, HLPlace> sourceTokens = new HashMap<Token, HLPlace>();

	private Map<Token, HLPlace> targetTokens = new HashMap<Token, HLPlace>();

	public FireSplitHLTransitionCommand() {
		setLabel("fire transition");
	}

	/**
	 * alle nicht zusammenhngenden Komponenten ein selektiertes {@link ObjectNet} 
	 * im einem Vorbereichstelle werden zerlegt und fr jede zusammenhangskomponente
	 * wird ein {@link ObjectNet} erzeugt. diese ObjectNets werden in allen Nachbereichstellen 
	 * kopiert und die ursprungliche {@link ObjectNet} wird gelscht
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		HLPlace sourceNetPlace = RONModelUtil.getSourceNetPlaces(transition)
				.iterator().next();
		HLPlace targetNetPlace = RONModelUtil.getTargetNetPlaces(transition)
				.iterator().next();

		ObjectNet sourceNet = (ObjectNet) RONModelUtil.getSelectedTokens(
				sourceNetPlace).iterator().next();
		sourceNet.setPlace(null);
		sourceTokens.put(sourceNet, sourceNetPlace);

		ObjectNet newNet = (ObjectNet) EcoreUtil.copy(sourceNet);
		Collection<ObjectNet> subNets = RONModelUtil.split(newNet);
		for (ObjectNet subNet : subNets) {
			subNet.setPlace(targetNetPlace);
			subNet.setSelected(true);
			targetTokens.put(subNet, targetNetPlace);
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		HLPlace place = null;
		for (Token token : sourceTokens.keySet()) {
			place = sourceTokens.get(token);
			token.setPlace(place);
		}
		for (Token token : targetTokens.keySet()) {
			token.setPlace(null);
		}
	}

	/** 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		HLPlace place = null;
		for (Token token : sourceTokens.keySet()) {
			token.setPlace(null);
		}
		for (Token token : targetTokens.keySet()) {
			place = targetTokens.get(token);
			token.setPlace(place);
		}
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		return true;
	}

	public void setTransition(HLTransition transition) {
		this.transition = transition;
	}
}
