package roneditor.ronview.commands;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;

import roneditor.ronview.model.RONModelUtil;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

public class FireStandardHLTransitionCommand extends Command {
	private HLTransition transition;

	private Map<Token, HLPlace> sourceTokens = new HashMap<Token, HLPlace>();

	private Map<Token, HLPlace> targetTokens = new HashMap<Token, HLPlace>();

	public FireStandardHLTransitionCommand() {
		setLabel("fire transition");
	}

	/**
	 * alle im Vorbereichstellen selektierten {@link ObjectNet ObjectNets} werden in einem {@link ObjectNet} 
	 * vereinigt und das resultierende {@link ObjectNet} wird in allen Nachbereichstellen Kopiert. die
	 * ursprunglichen {@link ObjectNet ObjectNets} werden gelscht.
	 * wenn ein {@link Rule} selektiert wird, dann wird sie gelscht und in alle Nachbereichstellen kopiert  
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		Collection<HLPlace> sourceNetPlaces = RONModelUtil
				.getSourceNetPlaces(transition);
		Collection<HLPlace> sourceRulePlaces = RONModelUtil
				.getSourceRulePlaces(transition);

		for (HLPlace netPlace : sourceNetPlaces) {
			Token token = RONModelUtil.getSelectedTokens(netPlace).iterator()
					.next();
			sourceTokens.put(token, netPlace);
			token.setPlace(null);
		}

		Rule rule = null;
		for (HLPlace rulePlace : sourceRulePlaces) {
			Token token = RONModelUtil.getSelectedTokens(rulePlace).iterator()
					.next();
			rule = (Rule) token;
			sourceTokens.put(token, rulePlace);
			token.setPlace(null);
		}

		Collection<HLPlace> targetNetPlaces = RONModelUtil
				.getTargetNetPlaces(transition);
		Collection<HLPlace> targetRulePlaces = RONModelUtil
				.getTargetRulePlaces(transition);
		ObjectNet newNet = null;
		Rule newRule = null;
		for (HLPlace place : targetNetPlaces) {
			newNet = RONModelUtil.union(sourceTokens.keySet());
			newNet.setPlace(place);
			newNet.setSelected(true);
			targetTokens.put(newNet, place);
		}
		for (HLPlace place : targetRulePlaces) {
			newRule = (Rule) EcoreUtil.copy(rule);
			newRule.setPlace(place);
			newRule.setSelected(true);
			targetTokens.put(newRule, place);
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		HLPlace place = null;
		for (Token token : sourceTokens.keySet()) {
			place = sourceTokens.get(token);
			token.setPlace(place);
		}
		for (Token token : targetTokens.keySet()) {
			token.setPlace(null);
		}
	}

	/** 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		HLPlace place = null;
		for (Token token : sourceTokens.keySet()) {
			token.setPlace(null);
		}
		for (Token token : targetTokens.keySet()) {
			place = targetTokens.get(token);
			token.setPlace(place);
		}
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		return true;
	}

	public void setTransition(HLTransition transition) {
		this.transition = transition;
	}
}
