package roneditor.ronview.commands;

import org.eclipse.gef.commands.Command;

import roneditor.ronview.model.RONModelUtil;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.RON;
/**
 * command zum ndern des {@link HLNode Sources} eines {@link HLArc}
 */
public class ReconnectHLSourceCommand extends Command {
	protected HLNode oldSource;

	protected HLNode newSource;

	protected HLArc hlArc;

	public ReconnectHLSourceCommand() {
		setLabel("reconnect source");
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (newSource == hlArc.getSource())
			return true;
		if (newSource == null || hlArc == null
				|| newSource.getClass() == hlArc.getTarget().getClass())
			return false;
		return RONModelUtil.getHLArc(newSource, hlArc.getTarget()) == null;
	}

	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		HLNode target = hlArc.getTarget();
		RON ron = hlArc.getRon();
		oldSource = hlArc.getSource();

		hlArc.setSource(null);
		hlArc.setTarget(null);
		hlArc.setRon(null);

		hlArc.setSource(newSource);
		hlArc.setTarget(target);
		hlArc.setRon(ron);
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		HLNode target = hlArc.getTarget();
		RON ron = hlArc.getRon();

		hlArc.setSource(null);
		hlArc.setTarget(null);
		hlArc.setRon(null);

		hlArc.setSource(oldSource);
		hlArc.setTarget(target);
		hlArc.setRon(ron);
	}

	public HLNode getNewSource() {
		return newSource;
	}

	public void setNewSource(HLNode newSource) {
		this.newSource = newSource;
	}

	public HLArc getHLArc() {
		return hlArc;
	}

	public void setHLArc(HLArc hlArc) {
		this.hlArc = hlArc;
	}
}
