package roneditor.ronview.edit.parts;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import roneditor.ronview.edit.policies.HLArcBendPointEditPolicy;
import roneditor.ronview.edit.policies.HLArcEditPolicy;
import roneditor.ronview.edit.policies.HLArcEndpointEditPolicy;
import roneditor.ronview.figures.HLArcFigure;
import ronmodel.BendPoint;
import ronmodel.HLArc;
import ronmodel.RonmodelPackage;

import java.util.ArrayList;
import java.util.List;

/**
 * EditPart for a hlArc.
 */
public class HLArcEditPart extends AbstractConnectionEditPart {

	private Notifier target;

	private HLArcAdapter adapter;

	public HLArcEditPart(HLArc hlArc) {
		super();
		setModel(hlArc);
	}

	public HLArc getHLArc() {
		return (HLArc) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,
				new HLArcEndpointEditPolicy());
		// installEditPolicy( EditPolicy.CONNECTION_BENDPOINTS_ROLE, new
		// ArcSelectionHandlesEditPolicy() );
		installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new HLArcBendPointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new HLArcEditPolicy());
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new HLArcAdapter();
		getHLArc().eAdapters().add(adapter);
		super.activate();
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		Connection c = (Connection) super.createFigure();
		return new HLArcFigure(c);
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		refreshBendpoints();
	}

	/**
	 * Hilfsfunktion fuer refreshVisuals()
	 * 
	 */
	private void refreshBendpoints() {
		// TODO: BendPoints erweitern
		List modelConstraint = getHLArc().getBendPoints();
		List figureConstraint = new ArrayList();
		if (modelConstraint != null) {
			for (int i = 0; i < modelConstraint.size(); i++) {
				BendPoint wbp = (BendPoint) modelConstraint.get(i);
				RelativeBendpoint rbp = new RelativeBendpoint(
						getConnectionFigure());
				Dimension first = new Dimension((Integer) wbp.getW().get(0),
						(Integer) wbp.getH().get(0));
				Dimension second;
				if(wbp.getW().size()==1){
					second = new Dimension((Integer) wbp.getW().get(0), (Integer) wbp.getH().get(1));
				}
				else{
					second = new Dimension((Integer) wbp.getW().get(1), (Integer) wbp.getH().get(1));
				}
				rbp.setRelativeDimensions(first, second);
				figureConstraint.add(rbp);
			}
			getConnectionFigure().setRoutingConstraint(figureConstraint);
		}
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class HLArcAdapter implements Adapter {
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.HL_ARC__BEND_POINTS:
				refreshBendpoints();
				break;
			}
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(type.getClass());
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class
}
