package roneditor.ronview.edit.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.ronview.commands.CreateHLArcCommand;
import roneditor.ronview.commands.ReconnectHLSourceCommand;
import roneditor.ronview.commands.ReconnectHLTargetCommand;
import ronmodel.HLArc;
import ronmodel.HLNode;

/**
 * EditPolicy for a {@link HLNode}.
 */
public class HLNodeEditPolicy extends GraphicalNodeEditPolicy {

	public HLNodeEditPolicy() {
		super();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(
			CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		if (command instanceof CreateHLArcCommand) {
			CreateHLArcCommand commandHLNode = (CreateHLArcCommand) command;
			commandHLNode.setTarget(getHLNode());
		}
		return command;
	}
	/**
	 * 
	 * @return the {@link HLNode}
	 */
	protected HLNode getHLNode() {
		return (HLNode) getHost().getModel();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		CreateHLArcCommand command = new CreateHLArcCommand();
		Object reqObj = request.getNewObject();
		if (reqObj instanceof HLArc) {
			command.setHlArc((HLArc) reqObj);
		}
		command.setSource(getHLNode());
		request.setStartCommand(command);
		return command;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		Object reqObj = request.getConnectionEditPart().getModel();
		ReconnectHLSourceCommand command = null;
		if (reqObj instanceof HLArc) {
			command = new ReconnectHLSourceCommand();
			command.setHLArc((HLArc) reqObj);
			command.setNewSource((HLNode) request.getTarget().getModel());
		}
		return command;
	}

	/** 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		Object reqObj = request.getConnectionEditPart().getModel();
		ReconnectHLTargetCommand command = null;
		if (reqObj instanceof HLArc) {
			command = new ReconnectHLTargetCommand();
			command.setHLArc((HLArc) reqObj);
			command.setNewTarget((HLNode) request.getTarget().getModel());
		}
		return command;
	}
}
