package roneditor.ronview.edit.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import roneditor.ronview.commands.CreateHLPlaceCommand;
import roneditor.ronview.commands.CreateHLTransitionCommand;
import roneditor.ronview.commands.SetConstraintCommand;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.RON;

/**
 * EditPolicy for XY-Layout.
 */
public class RONViewXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Object newObjectType = request.getNewObjectType();
		Command createCommand = null;

		if (newObjectType == HLPlace.class) {
			CreateHLPlaceCommand create = new CreateHLPlaceCommand(
					(RON) getHost().getModel(), (HLPlace) request
							.getNewObject());
			create.setX(request.getLocation().x);
			create.setY(request.getLocation().y);
			create.setLabel("create a HLPlace");
			createCommand = create;
		} else if (newObjectType == HLTransition.class) {
			CreateHLTransitionCommand create = new CreateHLTransitionCommand(
					(RON) getHost().getModel(), (HLTransition) request
							.getNewObject());
			create.setX(request.getLocation().x);
			create.setY(request.getLocation().y);
			create.setLabel("create a HLTransition");
			createCommand = create;
		}
		return createCommand;
	}

	/** 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/** 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		return new NonResizableEditPolicy();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getMinimumSizeFor(org.eclipse.gef.GraphicalEditPart)
	 */
	protected Dimension getMinimumSizeFor(GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}

	/** 
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(EditPart child,
			Object constraint) {
		SetConstraintCommand locationCommand = new SetConstraintCommand();
		locationCommand.setHLNode((HLNode) child.getModel());
		locationCommand.setNewPosition(((Rectangle) constraint).getLocation());
		return locationCommand;
	}

}
