package roneditor.ronview.figures;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import ronmodel.HLPlace;

/**
 * Represents the graphical layout of a {@link HLPlace}.
 */
public class HLPlaceFigure extends RectangleFigure {
	public static final int MIN_D = 36;

	public static final int TOKEN_DISTANCE = 4;

	protected Label label;

	protected XYLayout layout;

	private RoundedRectangle ellipse;

	private ConnectionAnchor connectionAnchor;

	public HLPlaceFigure() {
		super();
		layout = new XYLayout();
		setLayoutManager(layout);
		setOutline(false);
		ellipse = new RoundedRectangle() {
			@Override
			protected boolean useLocalCoordinates() {
				return true;
			}
		};
		ellipse.setSize(MIN_D, MIN_D);
		connectionAnchor = new ChopboxAnchor(ellipse);
		label = new Label("  ");
		add(ellipse);
		add(label);
	}
	/**
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		if (name == null) {
			label.setText("  ");
		} else {
			label.setText(" " + name + " ");
		}
		validate();
	}
	/**
	 * 
	 * @param figure the figure to add
	 * @param index the index of this figure
	 * @param children number ob children
	 */
	public void addToE(IFigure figure, int index, int children) {
		// if (children!=0) {
		int nbOfColumns = (int) Math.ceil(Math.sqrt(children));
		int nbOfRows = (int) Math.ceil(Double.valueOf(children) / nbOfColumns);
		int wToken = figure.getSize().width;
		int hToken = figure.getSize().height;
		int wPlace = wToken * nbOfColumns + (nbOfColumns + 1) * TOKEN_DISTANCE;
		int hPlace = hToken * nbOfRows + (nbOfRows + 1) * TOKEN_DISTANCE;
		wPlace = Math.max(wPlace, MIN_D);
		hPlace = Math.max(hPlace, MIN_D);
		ellipse.setSize(wPlace, hPlace);
		int r = index / nbOfColumns;
		int c = index % nbOfColumns;
		Point location = new Point(c * wToken + (c + 1) * TOKEN_DISTANCE, r
				* hToken + (r + 1) * TOKEN_DISTANCE);
		figure.setLocation(location);
		ellipse.add(figure, index);
		ellipse.validate();
		// }
	}
	/**
	 * remove All children from this Figure
	 */
	public void removeAllChildrenFromE() {
		ellipse.removeAll();
	}
	/**
	 * 
	 * @param figure the Figue to remove
	 */
	public void removeChildFromE(IFigure figure) {
		ellipse.remove(figure);
	}

	public void validate() {
		layout.setConstraint(ellipse, new Rectangle((getSize().width - ellipse
				.getSize().width) / 2, 1, -1, -1));
		layout.setConstraint(label, new Rectangle((getSize().width - label
				.getSize().width) / 2, ellipse.getSize().height + 5, -1, -1));
		super.validate();
	}

	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return getPreferredSize();
	}

	/** 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = Math.max(ellipse.getSize().width, label.getSize().width);
		int height = ellipse.getSize().height + label.getSize().height + 5;
		return new Dimension(width, height);
	}
	/**
	 * 
	 * @return the {@link ConnectionAnchor}
	 */
	public ConnectionAnchor getConnectionAnchor() {
		return connectionAnchor;
	}
}