package roneditor.ronview.figures;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
/**
 * Represents the graphical layout of a {@link HLTransition}.
 */
public class HLTransitionFigure extends RectangleFigure {
	public static final int NUMBER_OF_ANCHORS = 12;

	protected Label nameLabel;

	protected Label kindLabel;

	protected XYLayout layout;

	protected ConnectionAnchor connectionAnchor;

	public HLTransitionFigure() {
		super();
		layout = new XYLayout();
		setLayoutManager(layout);
		setBackgroundColor(ColorConstants.white);
		setOpaque(false);
		connectionAnchor = new ChopboxAnchor(this);
		nameLabel = new Label(" ");
		kindLabel = new Label(" ");
		add(nameLabel);
		add(kindLabel);
	}
	/**
	 * 
	 * @param name the name to set
	 */
	public void setName(String name) {
		if (name != null) {
			nameLabel.setText(name);
		} else {
			nameLabel.setText(" ");
		}

		validate();
	}
	/**
	 * 
	 * @param kind the kind to set
	 */
	public void setKind(HLTransitionKind kind) {
		kindLabel.setText("kind=" + kind);
		validate();
	}
	/**
	 * @see org.eclipse.draw2d.Figure#validate()
	 */
	public void validate() {
		layout.setConstraint(nameLabel, new Rectangle(5, 5, -1, -1));
		layout.setConstraint(kindLabel, new Rectangle(5, 5 + nameLabel
				.getBounds().height + 5, -1, -1));
		super.validate();
	}
	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	/**
	 * @see org.eclipse.draw2d.Figure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return getPreferredSize();
	}
	/**
	 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = Math.max(nameLabel.getSize().width,
				kindLabel.getSize().width) + 10;
		return new Dimension(width, nameLabel.getSize().height
				+ kindLabel.getSize().height + 15);
	}
	/**
	 * 
	 * @return the {@link ConnectionAnchor}
	 */
	public ConnectionAnchor getConnectionAnchor() {
		return connectionAnchor;
	}
	/**
	 * @see org.eclipse.draw2d.Figure#paintChildren(org.eclipse.draw2d.Graphics)
	 */
	@Override
	protected void paintChildren(Graphics graphics) {
		super.paintChildren(graphics);
		graphics.drawLine(0, getBounds().height / 2, getBounds().width - 1,
				getBounds().height / 2);
	}
}
