package roneditor.ronview.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;

/**
 * This class implements the CreationFactory used by the CreationTool. It in
 * turn uses the EMF-generated factories to create the model instances
 * 
 */
public class RONModelCreationFactory implements CreationFactory {
	private Class targetClass;

	public RONModelCreationFactory(Class targetClass) {
		this.targetClass = targetClass;
	}

	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String ronmodelURI = RonmodelPackage.eNS_URI;
		RonmodelPackage ronmodelPackage = (RonmodelPackage) registry
				.get(ronmodelURI);
		RonmodelFactory factory = ronmodelPackage.getRonmodelFactory();

		Object result = null;
		if (targetClass.equals(HLPlace.class)) {
			result = factory.createHLPlace();
		} else if (targetClass.equals(HLTransition.class)) {
			result = factory.createHLTransition();
		} else if (targetClass.equals(HLArc.class)) {
			result = factory.createHLArc();
		}
		return result;
	}

	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}
}
