package roneditor.ruleview.commands;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.gef.commands.Command;

import roneditor.ruleview.model.RuleModelCreationFactory;
import roneditor.ruleview.model.RuleModelUtil;
import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.RHS;
/**
 * command fr Kopieren eines {@link LHS} in ein {@link Codomain}
 */
public class CopyLHS2CodomainCommand extends Command {

	private static final String copyCommand_Label = "Copy LHS to Codomain";

	protected LHS lhs;

	protected Codomain codomain;

	/**
	 * Constructor.
	 */
	public CopyLHS2CodomainCommand() {
		super(copyCommand_Label);
	}

	/**
	 * @return true wenn das Ziel leer ist.false sonst
	 */
	public boolean canExecute() {
		if (lhs == null || codomain == null || !codomain.getNodes().isEmpty()
				|| lhs.getNodes().isEmpty())
			return false;

		return true;
	}

	/**
	 * Entry point for the command execution.
	 * der Inhalt von {@link LHS} wird in einem {@link Codomain} Kopiert
	 */
	public void execute() {
		Map<ONNode, ONNode> mappings = new HashMap<ONNode, ONNode>();
		ONNode onNode1 = null;
		ONNode onNode2 = null;
		Mapping mapping = null;
		RuleModelCreationFactory f = new RuleModelCreationFactory(Mapping.class);
		for (Object o : lhs.getNodes()) {
			onNode1 = (ONNode) o;
			onNode2 = RuleModelUtil.copy(onNode1);
			onNode2.setObjectNet(codomain);
			mappings.put(onNode1, onNode2);
			mapping = (Mapping) f.getNewObject();
			mapping.setSource(onNode1);
			mapping.setTarget(onNode2);
			mapping.setCodomain(codomain);
		}
		ONArc onArc1 = null;
		ONArc onArc2 = null;
		for (Object o : lhs.getArcs()) {
			onArc1 = (ONArc) o;
			onArc2 = RuleModelUtil.copy(onArc1);
			onArc2.setSource(mappings.get(onArc1.getSource()));
			onArc2.setTarget(mappings.get(onArc1.getTarget()));
			onArc2.setObjectNet(codomain);
		}
	}

	/**
	 * Returns the command label.
	 */
	public String getLabel() {
		if (codomain instanceof RHS) {
			return "Copy L->R";
		} else if (codomain instanceof NAC) {
			return "Copy L->NAC";
		}
		return super.getLabel();
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		Mapping mapping = null;
		List tmpList = new LinkedList(codomain.getMappings());
		for (Object o : tmpList) {
			mapping = (Mapping) o;
			mapping.setSource(null);
			mapping.setTarget(null);
			mapping.setCodomain(null);
		}
		ONArc arc = null;
		tmpList.clear();
		tmpList.addAll(codomain.getArcs());
		for (Object o : tmpList) {
			arc = (ONArc) o;
			arc.setSource(null);
			arc.setTarget(null);
			arc.setObjectNet(null);
		}
		ONNode node = null;
		tmpList.clear();
		tmpList.addAll(codomain.getNodes());
		for (Object o : tmpList) {
			node = (ONNode) o;
			node.setObjectNet(null);
		}
	}

	public void setCodomain(Codomain codomain) {
		this.codomain = codomain;
	}

	public void setLhs(LHS lhs) {
		this.lhs = lhs;
	}
}
