package roneditor.ruleview.commands;

import org.eclipse.gef.commands.Command;

import roneditor.ruleview.model.RuleModelUtil;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * Command class for a mappingcreation
 */
public class CreateMappingCommand extends Command {
	private static final String ConnectionCommand_Label = "Mapping";

	private static final String ConnectionCommand_Description = "Mapping: ONNode connection command";

	protected Mapping oldOutgoingMapping;

	protected Mapping oldIncomingMapping;

	protected ONNode source;

	protected ONNode target;

	protected Mapping mapping;

	protected Codomain codomain;

	protected DeleteMappingCommand deleteOldOutgoingMappingCommand;

	protected DeleteMappingCommand deleteOldIncomingMappingCommand;

	/**
	 * Constructor.
	 */
	public CreateMappingCommand() {
		super(ConnectionCommand_Label);
		setLabel("create Mapping");
		deleteOldOutgoingMappingCommand = new DeleteMappingCommand();
		deleteOldIncomingMappingCommand = new DeleteMappingCommand();
	}

	/**
	 * @return true wenn source und target vom gleichen Typ sind und wenn 
	 * keine Mapping zwischen source und target schon existiert. false sonst 
	 */
	public boolean canExecute() {
		if (source == null || target == null || mapping == null
				|| codomain == null || source.getClass() != target.getClass())
			return false;
		return RuleModelUtil.getMapping(source, target) == null;
	}

	/**
	 * Entry point for the command execution.
	 * Mapping wird zwischen source und target erzeugt
	 */
	public void execute() {
		oldIncomingMapping = RuleModelUtil.getIncomingMapping(target);
		oldOutgoingMapping = RuleModelUtil.getOutgoingMapping(source, codomain);
		deleteOldIncomingMappingCommand.setMapping(oldIncomingMapping);
		deleteOldOutgoingMappingCommand.setMapping(oldOutgoingMapping);
		if (deleteOldIncomingMappingCommand.canExecute()) {
			deleteOldIncomingMappingCommand.execute();
		}
		if (deleteOldOutgoingMappingCommand.canExecute()) {
			deleteOldOutgoingMappingCommand.execute();
		}
		mapping.setSource(source);
		mapping.setTarget(target);
		target.setMappingID(source.getMappingID());
		mapping.setCodomain(codomain);
	}

	/**
	 * Returns the command label.
	 */
	public String getLabel() {
		return ConnectionCommand_Description;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {

		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		mapping.setCodomain(null);
		mapping.setSource(null);
		mapping.setTarget(null);
		target.setMappingID(0);
		if (oldOutgoingMapping != null) {
			deleteOldOutgoingMappingCommand.undo();
		}
		if (oldIncomingMapping != null) {
			deleteOldIncomingMappingCommand.undo();
		}
	}

	public Codomain getCodomain() {
		return codomain;
	}

	public void setCodomain(Codomain codomain) {
		this.codomain = codomain;
	}

	public Mapping getMapping() {
		return mapping;
	}

	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	public ONNode getSource() {
		return source;
	}

	public void setSource(ONNode source) {
		this.source = source;
	}

	public ONNode getTarget() {
		return target;
	}

	public void setTarget(ONNode target) {
		this.target = target;
	}
}
