package roneditor.ruleview.commands;

import java.util.HashMap;

import org.eclipse.gef.commands.Command;

import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.Rule;
/**
 * Command for deleting a NAC.
 */
public class DeleteNACCommand extends Command {

	private static final String deleteCommand_Label = "delete NAC Command";

	protected Rule rule;

	protected NAC nac;

	protected HashMap<Mapping, ONNode> sourceMappings = new HashMap<Mapping, ONNode>();

	protected HashMap<Mapping, ONNode> targetMappings = new HashMap<Mapping, ONNode>();

	/**
	 * Constructor.
	 */
	public DeleteNACCommand() {
		super(deleteCommand_Label);
	}
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		rule = nac.getRule();
		deleteMappings();
		nac.setRule(null);
	}
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		nac.setRule(rule);
		addMappings();
	}
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {
		execute();
	}

	public void setNac(NAC nac) {
		this.nac = nac;
	}

	@Override
	public boolean canExecute() {
		return nac != null && nac.getRule().getNacs().size() > 1;
	}

	protected void deleteMappings() {
		Mapping mapping = null;
		for (Object o : nac.getMappings()) {
			mapping = (Mapping) o;
			sourceMappings.put(mapping, mapping.getSource());
			targetMappings.put(mapping, mapping.getTarget());
		}
		for (Mapping mapping2 : sourceMappings.keySet()) {
			mapping2.setSource(null);
			mapping2.setTarget(null);
			mapping2.setCodomain(null);
		}
	}

	protected void addMappings() {
		for (Mapping mapping : sourceMappings.keySet()) {
			mapping.setSource(sourceMappings.get(mapping));
			mapping.setTarget(targetMappings.get(mapping));
			mapping.setCodomain(nac);
		}
		sourceMappings.clear();
		targetMappings.clear();
	}

}
