package roneditor.ruleview.edit.parts;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import ronmodel.Mapping;
import ronmodel.RonmodelPackage;
/**
 * editpart for a mapping
 */
public class MappingEditPart extends AbstractConnectionEditPart {
	private Notifier target;

	private MappingAdapter adapter;

	
	public MappingEditPart(Mapping mapping) {
		super();
		setModel(mapping);
	}

	public Mapping getMapping() {
		return (Mapping) getModel();
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		

	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		adapter = new MappingAdapter();
		getMapping().eAdapters().add(adapter);
		super.activate();
	}

	/**
	 * This local class is needed because ConnectionEditParts already define a
	 * getTarget() method which returns an EditPart.
	 */
	private class MappingAdapter implements Adapter {
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {
			int featureId = notification.getFeatureID(RonmodelPackage.class);
			switch (featureId) {
			case RonmodelPackage.MAPPING__SOURCE:
			case RonmodelPackage.MAPPING__TARGET:
				refreshVisuals();
				break;
			}
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return target;
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterFor(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return type.equals(type.getClass());
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			target = newTarget;
		}

	}// inner class
}
